"""
This script provides access to a set of utilities for Wiki-Labels

* dev_server -- Starts a development webserver on localhost
* load_schema -- Loads the DB schema
* new_campaign -- Creates a new campaign
* task_inserts -- Inserts a set of tasks into a campaign
* remove_expired_tasks -- Removes expired tasks from worksets

Usage:
    wikilabels (-h | --help)
    wikilabels <utility> [-h | --help]

Options:
    -h | --help  Shows this documentation
    <utility>    The name of the utility to run
"""
import sys
import traceback
from importlib import import_module


USAGE = """Usage:
    wikilabels (-h | --help)
    wikilabels <utility> [-h | --help]\n"""


def main():

    if len(sys.argv) < 2:
        sys.stderr.write(USAGE)
        sys.exit(1)
    elif sys.argv[1] in ("-h", "--help"):
        sys.stderr.write(__doc__ + "\n")
        sys.exit(1)
    elif sys.argv[1][:1] == "-":
        sys.stderr.write(USAGE)
        sys.exit(1)

    module_name = sys.argv[1]
    try:
        module = import_module(".utilities." + module_name,
                               package="wikilabels")
    except ImportError:
        sys.stderr.write(traceback.format_exc())
        sys.stderr.write("Could not find utility {0}.\n".format(module_name))
        sys.exit(1)

    module.main(sys.argv[2:])
